"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EditorSettings = void 0;
const tslib_1 = require("tslib");
/* eslint-disable jsx-a11y/no-static-element-interactions */
/* eslint-disable jsx-a11y/click-events-have-key-events */
const react_1 = tslib_1.__importDefault(require("react"));
const components_1 = require("@gravity-ui/components");
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../../classname");
const bundle_1 = require("../../i18n/bundle");
const WysiwygMode_1 = tslib_1.__importDefault(require("../../icons/WysiwygMode"));
const lodash_1 = require("../../lodash");
const hooks_1 = require("../../react-utils/hooks");
const toolbar_1 = require("../../toolbar");
const version_1 = require("../../version");
const MarkdownHints_1 = require("./MarkdownHints");
const placement = ['bottom-end', 'top-end'];
const bSettings = (0, classname_1.cn)('editor-settings');
const bContent = (0, classname_1.cn)('settings-content');
exports.EditorSettings = react_1.default.memo(function EditorSettings(props) {
    const { className, onShowPreviewChange, showPreview, renderPreviewButton, settingsVisible } = props;
    const chevronRef = react_1.default.useRef(null);
    const [popupShown, , hidePopup, togglePopup] = (0, hooks_1.useBooleanState)(false);
    return (react_1.default.createElement("div", { className: bSettings(null, [className]) },
        renderPreviewButton && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(uikit_1.ActionTooltip, { openDelay: toolbar_1.ToolbarTooltipDelay.Open, closeDelay: toolbar_1.ToolbarTooltipDelay.Close, title: (0, bundle_1.i18n)('preview_hint'), hotkey: "mod+shift+p" },
                react_1.default.createElement(uikit_1.Button, { size: "m", view: "flat", pin: "round-round", className: bSettings('preview-button'), onClick: () => onShowPreviewChange === null || onShowPreviewChange === void 0 ? void 0 : onShowPreviewChange(!showPreview), selected: showPreview },
                    react_1.default.createElement(uikit_1.Icon, { data: icons_1.Eye }))),
            settingsVisible && react_1.default.createElement("div", { className: bSettings('separator') }))),
        settingsVisible && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(uikit_1.Button, { size: "m", view: "flat", ref: chevronRef, pin: "round-round", onClick: togglePopup, className: bSettings('dropdown-button') },
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.Gear })),
            react_1.default.createElement(uikit_1.Popup, { open: popupShown, anchorRef: chevronRef, placement: placement, onClose: hidePopup },
                react_1.default.createElement(SettingsContent, Object.assign({}, props, { onClose: hidePopup, className: bSettings('content') })))))));
});
const mdHelpPlacement = ['bottom', 'bottom-end', 'right-start', 'right', 'left'];
const SettingsContent = function SettingsContent({ mode, onClose, onModeChange, toolbarVisibility, onToolbarVisibilityChange, onSplitModeChange, splitMode = false, splitModeEnabled, className, showPreview, }) {
    return (react_1.default.createElement("div", { className: bContent(null, [className]) },
        react_1.default.createElement(uikit_1.Menu, { size: "l", className: bContent('mode') },
            react_1.default.createElement(uikit_1.Menu.Item, { active: mode === 'wysiwyg', onClick: () => {
                    onModeChange('wysiwyg');
                    onClose();
                }, icon: react_1.default.createElement(uikit_1.Icon, { data: WysiwygMode_1.default }) }, (0, bundle_1.i18n)('settings_wysiwyg')),
            react_1.default.createElement(uikit_1.Menu.Item, { active: mode === 'markup', onClick: () => {
                    onModeChange('markup');
                    onClose();
                }, icon: react_1.default.createElement(uikit_1.Icon, { data: icons_1.LogoMarkdown }) },
                (0, bundle_1.i18n)('settings_markup'),
                react_1.default.createElement(components_1.HelpPopover, { content: react_1.default.createElement("div", { onClick: (e) => {
                            // stop clicks propagation
                            // because otherwise click in MarkdownHints handled as click on MenuItem
                            e.stopPropagation();
                        } },
                        react_1.default.createElement(MarkdownHints_1.MarkdownHints, null)), placement: mdHelpPlacement, className: bContent('mode-help') }))),
        react_1.default.createElement("div", { className: bContent('separator') }),
        !showPreview && (react_1.default.createElement("div", { className: bContent('toolbar') },
            react_1.default.createElement(uikit_1.Checkbox, { size: "m", checked: toolbarVisibility, onUpdate: onToolbarVisibilityChange }, (0, bundle_1.i18n)('settings_menubar')),
            react_1.default.createElement("div", { className: bContent('toolbar-hint') }, (0, bundle_1.i18n)('settings_hint')))),
        splitMode && (react_1.default.createElement("div", { className: bContent('split-mode') },
            react_1.default.createElement(uikit_1.Checkbox, { size: "m", disabled: mode !== 'markup', checked: splitModeEnabled, onUpdate: onSplitModeChange !== null && onSplitModeChange !== void 0 ? onSplitModeChange : lodash_1.noop }, (0, bundle_1.i18n)('settings_split-mode')),
            react_1.default.createElement("div", { className: bContent('toolbar-hint') }, (0, bundle_1.i18n)('settings_split-mode-hint')))),
        react_1.default.createElement("span", { className: bContent('version') }, version_1.VERSION)));
};
