"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolbarColors = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const classname_1 = require("../../../classname");
const extensions_1 = require("../../../extensions");
const menubar_1 = require("../../../i18n/menubar");
const icons_1 = require("../../config/icons");
const ToolbarButtonWithPopupMenu_1 = require("../ToolbarButtonWithPopupMenu");
const b = (0, classname_1.cn)('toolbar-colors');
const textColorIcon = icons_1.icons.textColor;
const ToolbarColors = (props) => {
    var _a;
    const { exec, onClick, enable, currentColor, withDefault } = props;
    const isDefault = currentColor === undefined;
    const onItemClick = (color) => () => {
        // do not exec when current color is default and clicked to default item
        if (!(isDefault && color === ''))
            exec(color);
        onClick === null || onClick === void 0 ? void 0 : onClick('colorify', { color: color === '' ? 'default' : color });
    };
    const items = [
        extensions_1.Colors.Gray,
        extensions_1.Colors.Yellow,
        extensions_1.Colors.Orange,
        extensions_1.Colors.Red,
        extensions_1.Colors.Green,
        extensions_1.Colors.Blue,
        extensions_1.Colors.Violet,
    ].map((color) => {
        var _a;
        return ({
            id: color,
            icon: textColorIcon.data,
            iconSize: (_a = textColorIcon.size) !== null && _a !== void 0 ? _a : 16,
            text: (0, menubar_1.i18n)(`colorify__color_${color}`),
            iconClassname: b('item-icon', { color }),
            action: {
                run: onItemClick(color),
                isEnable: () => Boolean(enable),
                isActive: () => color === currentColor,
                meta() { },
            },
            group: (0, menubar_1.i18n)(`colorify__group_text`),
        });
    });
    if (withDefault) {
        items.unshift({
            id: 'default',
            icon: textColorIcon.data,
            iconSize: (_a = textColorIcon.size) !== null && _a !== void 0 ? _a : 16,
            text: (0, menubar_1.i18n)(`colorify__color_default`),
            iconClassname: b('item-icon', { color: 'default' }),
            group: (0, menubar_1.i18n)(`colorify__group_text`),
            ignoreActive: true,
            action: {
                run: onItemClick(''),
                isEnable: () => Boolean(enable),
                isActive: () => isDefault,
                meta() { },
            },
        });
    }
    return (react_1.default.createElement(ToolbarButtonWithPopupMenu_1.ToolbarButtonWithPopupMenu, Object.assign({}, props, { title: (0, menubar_1.i18n)('colorify'), menuItems: items, icon: textColorIcon })));
};
exports.ToolbarColors = ToolbarColors;
