import type MarkdownIt from 'markdown-it';
import type OrderedMap from 'orderedmap';
import type { MarkSpec, NodeSpec, Schema } from 'prosemirror-model';
import type { Plugin } from 'prosemirror-state';
import type { ExtensionBuilder } from '../ExtensionBuilder';
import type { ActionSpec, ActionStorage } from './actions';
import type { MarkViewConstructor, NodeViewConstructor } from './node-views';
import type { Parser, ParserToken } from './parser';
import type { Serializer, SerializerMarkToken, SerializerNodeToken } from './serializer';
export declare type Extension = (builder: ExtensionBuilder) => void;
export declare type ExtensionWithOptions<T> = (builder: ExtensionBuilder, options: T) => void;
export declare type ExtensionAuto<T = void> = T extends void ? Extension : ExtensionWithOptions<T>;
export declare type ExtensionSpec = {
    configureMd(md: MarkdownIt, parserType: 'text' | 'markup'): MarkdownIt;
    nodes(): OrderedMap<ExtensionNodeSpec>;
    marks(): OrderedMap<ExtensionMarkSpec>;
    plugins(deps: ExtensionDeps): Plugin[];
    actions(deps: ExtensionDeps): Record<string, ActionSpec>;
};
export declare type ExtensionNodeSpec = {
    spec: NodeSpec;
    view?: (deps: ExtensionDeps) => NodeViewConstructor;
    fromMd: {
        tokenName?: string;
        tokenSpec: ParserToken;
    };
    toMd: SerializerNodeToken;
};
export declare type ExtensionMarkSpec = {
    spec: MarkSpec;
    view?: (deps: ExtensionDeps) => MarkViewConstructor;
    fromMd: {
        tokenName?: string;
        tokenSpec: ParserToken;
    };
    toMd: SerializerMarkToken;
};
export declare type ExtensionDeps = {
    readonly schema: Schema;
    readonly textParser: Parser;
    readonly markupParser: Parser;
    readonly serializer: Serializer;
    readonly actions: ActionStorage;
};
