"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GptDialog = exports.cnGptDialog = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../../../../classname");
const dialog_1 = require("../../../../i18n/gpt/dialog");
const GPT_1 = tslib_1.__importDefault(require("../../../../icons/GPT"));
const useAutoFocus_1 = require("../../../../react-utils/useAutoFocus");
const ErrorScreen_1 = require("../ErrorScreen/ErrorScreen");
const IconRefuge_1 = require("../IconRefuge/IconRefuge");
const PresetList_1 = require("../PresetList/PresetList");
const constants_1 = require("../constants");
const useGpt_1 = require("../hooks/useGpt");
const useGptHotKeys_1 = require("../hooks/useGptHotKeys");
const utils_1 = require("../utils");
const LoadingScreen_1 = require("./LoadingScreen/LoadingScreen");
exports.cnGptDialog = (0, classname_1.cn)('gpt-dialog');
const GptDialog = ({ markup, answerRender, promptPresets, disablePromptPresets, customPromptPlaceholder, disabledPromptPlaceholder, onCustomPromptApply, onPromptPresetClick, onTryAgain, onApplyResult, onClose, onLike, onDislike, onUpdate, gptAlertProps, }) => {
    const { answer, customPrompt, loading, mode, feedbackType, feedbackTypeLoading, handleLike, handleDislike, handleCustomPromptUpdate, handleCustomPromptKeyPress, handleCustomPromptApply, handlePresetClick, handleTryAgain, handleFreshStart, handleApplyResult, showAnswer, showError, showAnswerActions, showTryAgainButton, } = (0, useGpt_1.useGpt)({
        markup,
        promptPresets,
        onLike,
        onDislike,
        onCustomPromptApply,
        onPromptPresetClick,
        onTryAgain,
        onApplyResult,
        onUpdate,
    });
    const gptAlert = gptAlertProps;
    const customPromptContainerRef = (0, react_1.useRef)(null);
    const [showedGptAlert, setShowedGptAlert] = (0, react_1.useState)(gptAlert === null || gptAlert === void 0 ? void 0 : gptAlert.showedGptAlert);
    const { alertMessage, alertTheme, alertClassName } = (0, utils_1.getAlertGptInfo)(gptAlert);
    const onCloseAlert = (0, react_1.useCallback)(() => {
        var _a;
        if (gptAlert) {
            gptAlert.showedGptAlert = false;
        }
        (_a = gptAlert === null || gptAlert === void 0 ? void 0 : gptAlert.onCloseGptAlert) === null || _a === void 0 ? void 0 : _a.call(gptAlert);
        setShowedGptAlert(false);
    }, [gptAlert]);
    (0, useAutoFocus_1.useAutoFocus)(customPromptContainerRef, [showAnswer]);
    (0, useGptHotKeys_1.useGptHotKeys)(constants_1.gptHotKeys.tryAgainGpt, handleTryAgain);
    (0, useGptHotKeys_1.useGptHotKeys)(constants_1.gptHotKeys.freshStartGpt, handleFreshStart);
    (0, useGptHotKeys_1.useGptHotKeys)(constants_1.gptHotKeys.applyResultGpt, handleApplyResult);
    const replaceButtonText = (0, utils_1.getDisableReplaceButtonText)(disablePromptPresets);
    const inputPlaceholderText = (0, utils_1.getInputPlaceHolder)(disablePromptPresets, disabledPromptPlaceholder, customPromptPlaceholder);
    const tryAgainButton = (react_1.default.createElement(uikit_1.ActionTooltip, { hotkey: constants_1.gptHotKeys.tryAgainGpt, title: (0, dialog_1.i18n)('try-again') },
        react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('try-again-button'), view: "normal", size: "m", onClick: handleTryAgain },
            react_1.default.createElement(uikit_1.Icon, { data: icons_1.ArrowRotateLeft, size: 16 }),
            (0, dialog_1.i18n)('try-again'))));
    let content = null;
    if (loading) {
        content = react_1.default.createElement(LoadingScreen_1.LoadingScreen, null);
    }
    else if (showError) {
        content = react_1.default.createElement(ErrorScreen_1.ErrorScreen, { onRetry: handleTryAgain, onStartAgain: handleFreshStart });
    }
    else {
        content = (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('header') },
                react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('header-top') },
                    react_1.default.createElement(IconRefuge_1.IconRefuge, { containerClassName: (0, exports.cnGptDialog)('gpt-icon'), data: GPT_1.default, refugeSize: 28, size: 16 }),
                    (mode === 'only-custom' || mode === 'custom-and-presets') &&
                        (showAnswer ? (react_1.default.createElement("span", { className: (0, exports.cnGptDialog)('answer-title') }, (0, dialog_1.i18n)('answer-title'))) : (react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('custom-prompt') },
                            react_1.default.createElement(uikit_1.TextInput, { controlRef: customPromptContainerRef, view: "clear", size: "m", className: (0, exports.cnGptDialog)('custom-prompt-input'), placeholder: inputPlaceholderText, onKeyPress: handleCustomPromptKeyPress, onUpdate: handleCustomPromptUpdate }),
                            react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('custom-prompt-ask-button'), view: "normal", size: "s", disabled: !customPrompt, onClick: handleCustomPromptApply },
                                react_1.default.createElement(uikit_1.Icon, { data: icons_1.ArrowRight, size: 16 }))))),
                    mode === 'only-presets' && (react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('alone-presets') },
                        react_1.default.createElement("span", { className: (0, exports.cnGptDialog)('alone-presets-text') }, (0, dialog_1.i18n)('only-presets-title'))))),
                (mode === 'custom-and-presets' || mode === 'only-presets') &&
                    ((showTryAgainButton && (react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('header-bottom') },
                        tryAgainButton,
                        react_1.default.createElement(uikit_1.ActionTooltip, { title: (0, dialog_1.i18n)('fresh-start-button'), hotkey: constants_1.gptHotKeys.freshStartGpt },
                            react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('back-to-start'), view: "normal", size: "m", onClick: handleFreshStart }, (0, dialog_1.i18n)('fresh-start-button')))))) ||
                        (!disablePromptPresets && (react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('header-bottom') },
                            react_1.default.createElement(PresetList_1.PresetList, { disablePromptPresets: disablePromptPresets, promptPresets: promptPresets, onPresetClick: handlePresetClick })))) ||
                        (disablePromptPresets && showedGptAlert && (react_1.default.createElement(uikit_1.Alert, { theme: alertTheme, message: alertMessage, align: "center", className: (0, exports.cnGptDialog)('description-alert', alertClassName), onClose: onCloseAlert }))))),
            showAnswer && (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('body') },
                    react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('answer') }, answerRender(answer))),
                react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('footer') },
                    showAnswerActions && (react_1.default.createElement("div", { className: (0, exports.cnGptDialog)('answer-actions') },
                        react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('like-button'), view: "normal", size: "m", onClick: handleLike, selected: feedbackType === 'like', loading: feedbackTypeLoading === 'like' },
                            react_1.default.createElement(uikit_1.Icon, { data: icons_1.ThumbsUp, size: 16 })),
                        react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('dislike-button'), view: "normal", size: "m", onClick: handleDislike, selected: feedbackType === 'dislike', loading: feedbackTypeLoading === 'dislike' },
                            react_1.default.createElement(uikit_1.Icon, { data: icons_1.ThumbsDown, size: 16 })),
                        feedbackType && (react_1.default.createElement("span", { className: (0, exports.cnGptDialog)('feedback-message') }, (0, dialog_1.i18n)('feedback-message'))))),
                    react_1.default.createElement(uikit_1.ActionTooltip, { title: (0, dialog_1.i18n)('close-button'), hotkey: constants_1.gptHotKeys.closeGpt },
                        react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('close-button'), view: "flat", size: "m", onClick: onClose }, (0, dialog_1.i18n)('close-button'))),
                    react_1.default.createElement(uikit_1.ActionTooltip, { title: replaceButtonText, hotkey: constants_1.gptHotKeys.applyResultGpt },
                        react_1.default.createElement(uikit_1.Button, { className: (0, exports.cnGptDialog)('apply-button'), view: "action", size: "m", onClick: handleApplyResult }, replaceButtonText)))))));
    }
    return react_1.default.createElement("div", { className: (0, exports.cnGptDialog)() },
        " ",
        content,
        " ");
};
exports.GptDialog = GptDialog;
