"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Heading = exports.hType = exports.HeadingAction = exports.headingType = exports.headingNodeName = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const keymap_1 = require("../../../utils/keymap");
const HeadingSpecs_1 = require("./HeadingSpecs");
const actions_1 = require("./actions");
const commands_1 = require("./commands");
const const_1 = require("./const");
const utils_1 = require("./utils");
var HeadingSpecs_2 = require("./HeadingSpecs");
Object.defineProperty(exports, "headingNodeName", { enumerable: true, get: function () { return HeadingSpecs_2.headingNodeName; } });
Object.defineProperty(exports, "headingType", { enumerable: true, get: function () { return HeadingSpecs_2.headingType; } });
var const_2 = require("./const");
Object.defineProperty(exports, "HeadingAction", { enumerable: true, get: function () { return const_2.HeadingAction; } });
var utils_2 = require("./utils");
Object.defineProperty(exports, "hType", { enumerable: true, get: function () { return utils_2.hType; } });
const Heading = (builder, opts) => {
    builder.use(HeadingSpecs_1.HeadingSpecs, opts);
    builder
        .addKeymap(({ schema }) => {
        const { h1Key, h2Key, h3Key, h4Key, h5Key, h6Key } = opts !== null && opts !== void 0 ? opts : {};
        const cmd4lvl = (level) => (0, prosemirror_commands_1.setBlockType)((0, utils_1.hType)(schema), { [const_1.headingLevelAttr]: level });
        const bindings = { Backspace: commands_1.resetHeading };
        if (h1Key)
            bindings[h1Key] = (0, keymap_1.withLogAction)('heading1', cmd4lvl(1));
        if (h2Key)
            bindings[h2Key] = (0, keymap_1.withLogAction)('heading2', cmd4lvl(2));
        if (h3Key)
            bindings[h3Key] = (0, keymap_1.withLogAction)('heading3', cmd4lvl(3));
        if (h4Key)
            bindings[h4Key] = (0, keymap_1.withLogAction)('heading4', cmd4lvl(4));
        if (h5Key)
            bindings[h5Key] = (0, keymap_1.withLogAction)('heading5', cmd4lvl(5));
        if (h6Key)
            bindings[h6Key] = (0, keymap_1.withLogAction)('heading6', cmd4lvl(6));
        return bindings;
    })
        .addInputRules(({ schema }) => ({ rules: [(0, utils_1.headingRule)((0, utils_1.hType)(schema), 6)] }));
    builder
        .addAction(const_1.HeadingAction.ToH1, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 1))
        .addAction(const_1.HeadingAction.ToH2, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 2))
        .addAction(const_1.HeadingAction.ToH3, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 3))
        .addAction(const_1.HeadingAction.ToH4, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 4))
        .addAction(const_1.HeadingAction.ToH5, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 5))
        .addAction(const_1.HeadingAction.ToH6, ({ schema }) => (0, actions_1.headingAction)((0, utils_1.hType)(schema), 6));
};
exports.Heading = Heading;
