"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageNodeView = exports.cnImgSizeNodeView = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const classname_1 = require("../../../../../classname");
const react_utils_1 = require("../../../../../react-utils");
const useNodeResizing_1 = require("../../../../../react-utils/useNodeResizing");
const utils_1 = require("../../../../../utils");
const Resizable_1 = require("../../../../behavior/Resizable/Resizable");
const ImgSizeSpecs_1 = require("../../ImgSizeSpecs");
const const_1 = require("../../const");
const ImgSettingsButton_1 = require("./ImgSettingsButton");
exports.cnImgSizeNodeView = (0, classname_1.cn)('img-size-node-view');
const ImageNodeView = ({ node, view, getPos, updateAttributes, }) => {
    const imageContainerRef = (0, react_1.useRef)(null);
    const imageRef = (0, react_1.useRef)(null);
    const alt = node.attrs[ImgSizeSpecs_1.ImgSizeAttr.Alt] || '';
    const initialHeight = node.attrs[ImgSizeSpecs_1.ImgSizeAttr.Height];
    const initialWidth = node.attrs[ImgSizeSpecs_1.ImgSizeAttr.Width];
    const src = node.attrs[ImgSizeSpecs_1.ImgSizeAttr.Src] || '';
    const title = node.attrs[ImgSizeSpecs_1.ImgSizeAttr.Title] || '';
    const isNodeHovered = (0, react_utils_1.useNodeHovered)(imageContainerRef);
    const [edit, setEditing, unsetEdit, toggleEdit] = (0, react_utils_1.useNodeEditing)({
        nodeRef: imageContainerRef,
        view,
    });
    const handleResize = (0, react_1.useCallback)(({ width, height }) => {
        updateAttributes({
            width: width === undefined ? undefined : String(Math.round(width)),
            height: height === undefined ? undefined : String(Math.round(height)),
            name: title,
            alt,
        });
    }, [alt, title, updateAttributes]);
    const { state, startResizing } = (0, useNodeResizing_1.useNodeResizing)({
        width: initialWidth,
        height: initialHeight,
        ref: imageRef,
        onResize: handleResize,
    });
    const style = {
        width: state.width ? `${state.width}px` : '',
        height: state.height ? `${state.height}px` : '',
        transition: 'width 0.15s ease-out, height 0.15s ease-out',
    };
    const handleDelete = (0, react_1.useCallback)(() => {
        const pos = getPos();
        if (pos === undefined)
            return;
        (0, utils_1.removeNode)({
            node,
            pos,
            tr: view.state.tr,
            dispatch: view.dispatch,
        });
        view.focus();
    }, [getPos, node, view]);
    const createHandleResize = (direction) => (event) => {
        startResizing(event, direction);
    };
    (0, react_1.useEffect)(() => {
        var _a;
        if ((_a = const_1.imageRendererKey.getState(view.state)) === null || _a === void 0 ? void 0 : _a.linkAdded) {
            setEditing();
        }
    }, [view, setEditing]);
    return (react_1.default.createElement("div", { ref: imageContainerRef },
        react_1.default.createElement(Resizable_1.Resizable, { hover: isNodeHovered, resizing: state.resizing, onResizeLeft: createHandleResize('left'), onResizeRight: createHandleResize('right') },
            react_1.default.createElement(ImgSettingsButton_1.ImgSettingsButton, { node: node, view: view, getPos: getPos, updateAttributes: updateAttributes, visible: isNodeHovered && !edit && !state.resizing, edit: edit, toggleEdit: toggleEdit, nodeRef: imageRef, onDelete: handleDelete, unsetEdit: unsetEdit }),
            react_1.default.createElement("img", { ref: imageRef, src: src, alt: alt, style: style }))));
};
exports.ImageNodeView = ImageNodeView;
