"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImageSize = exports.loadImage = exports.createImageNode = exports.isImageNode = void 0;
const logger_1 = require("../../../logger");
const markdown_1 = require("../../markdown");
const specs_1 = require("../../specs");
const const_1 = require("./const");
function isImageNode(node) {
    return node.type.name === markdown_1.imageNodeName;
}
exports.isImageNode = isImageNode;
const createImageNode = (imgType, opts) => async ({ result, file }) => {
    var _a;
    const attrs = {
        [specs_1.ImgSizeAttr.Src]: result.url,
        [specs_1.ImgSizeAttr.Alt]: (_a = result.name) !== null && _a !== void 0 ? _a : file.name,
    };
    if (opts.needDimmensions) {
        try {
            const sizes = await loadImage(file).then(getImageSize);
            Object.assign(attrs, sizes);
        }
        catch (err) {
            logger_1.logger.error(err);
        }
    }
    return imgType.create(attrs);
};
exports.createImageNode = createImageNode;
async function loadImage(imgFile) {
    return new Promise((resolve, reject) => {
        const img = document.createElement('img');
        img.src = URL.createObjectURL(imgFile);
        img.onload = () => {
            URL.revokeObjectURL(img.src);
            resolve(img);
        };
        img.onerror = (_e, _s, _l, _c, error) => reject(error);
    });
}
exports.loadImage = loadImage;
function getImageSize(img) {
    return { height: String(Math.min(const_1.IMG_MAX_HEIGHT, img.height)) };
}
exports.getImageSize = getImageSize;
