"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilesUploadPlugin = void 0;
const view_1 = require("@codemirror/view");
const files_upload_facet_1 = require("../files-upload-facet");
const react_facet_1 = require("../react-facet");
const const_1 = require("./const");
const effects_1 = require("./effects");
const utils_1 = require("./utils");
const widget_1 = require("./widget");
class FileUploadWidget extends view_1.WidgetType {
    constructor(id) {
        super();
        this.renderer = null;
        this.props = {};
        this.selfId = id;
    }
    toDOM(view) {
        var _a;
        const dom = document.createElement('div');
        dom.className = 'cm-file-upload-widget';
        dom.style.display = 'inline-block';
        (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
        this.createRenderItem(dom, view);
        return dom;
    }
    updateDOM(dom, view) {
        var _a;
        (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
        this.createRenderItem(dom, view);
        return true;
    }
    destroy(dom) {
        var _a, _b;
        super.destroy(dom);
        (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
        (_b = this.presenter) === null || _b === void 0 ? void 0 : _b.cancel();
        this.presenter = undefined;
    }
    setPresenter(presenter) {
        this.presenter = presenter;
    }
    render(props) {
        var _a;
        Object.assign(this.props, props);
        (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.rerender();
    }
    createRenderItem(dom, view) {
        this.renderer = view.state
            .facet(react_facet_1.ReactRendererFacet)
            .createItem('cm-file-upload-widget', () => (0, widget_1.renderWidget)(dom, Object.assign({}, this.props)));
    }
}
class FileUploadPresenter {
    constructor(params) {
        this.state = 'initial';
        this.onUploadSuccess = (res) => {
            if (this.state === 'canceled')
                return;
            this.state = 'success';
            this.widget.render({
                status: 'success',
                fileName: this.file.name,
                fileType: (0, utils_1.isImageFile)(this.file) ? 'image' : 'file',
            });
            setTimeout(async () => {
                const markup = await this.formatFileMarkup(res);
                if (this.state === 'canceled')
                    return;
                this.view.dispatch({
                    effects: effects_1.RemoveUploadWidgetEffect.of({
                        id: this.widget.selfId,
                        markup,
                    }),
                });
            }, const_1.SUCCESS_UPLOAD_REMOVE_TIMEOUT);
        };
        this.onUploadError = (err) => {
            if (this.state === 'canceled')
                return;
            this.state = 'error';
            this.widget.render({
                status: 'error',
                fileName: this.file.name,
                fileType: (0, utils_1.isImageFile)(this.file) ? 'image' : 'file',
                errorText: String(err),
                onReUploadClick: () => {
                    if (this.state === 'error')
                        this.run();
                },
            });
        };
        this.file = params.file;
        this.view = params.view;
        this.widget = params.widget;
        this.uploader = params.uploader;
        this.needDimmensionsForImages = params.needDimmensionsForImages;
        this.widget.setPresenter(this);
        this.run();
    }
    cancel() {
        this.state = 'canceled';
    }
    run() {
        this.state = 'uploading';
        this.widget.render({
            status: 'uploading',
            fileName: this.file.name,
            fileType: (0, utils_1.isImageFile)(this.file) ? 'image' : 'file',
        });
        this.uploader(this.file).then(this.onUploadSuccess, this.onUploadError);
    }
    async formatFileMarkup(res) {
        var _a, _b;
        const fileName = (_b = (_a = res.name) !== null && _a !== void 0 ? _a : this.file.name) !== null && _b !== void 0 ? _b : '';
        let markup;
        if ((0, utils_1.isImageFile)(this.file)) {
            if (this.needDimmensionsForImages) {
                try {
                    let { height } = await (0, utils_1.getImageDimensions)(this.file);
                    height = Math.min(height, const_1.IMG_MAX_HEIGHT);
                    markup = `![${fileName}](${res.url} =x${height})`;
                }
                catch (err) {
                    markup = `![${fileName}](${res.url})`;
                }
            }
            else {
                markup = `![${fileName}](${res.url})`;
            }
        }
        else {
            markup = `{% file src="${res.url}" name="${fileName.replace('"', '')}" %}`;
        }
        return markup;
    }
}
exports.FilesUploadPlugin = view_1.ViewPlugin.fromClass(class {
    constructor(view) {
        this.decos = view_1.Decoration.none;
        this.view = view;
    }
    update(update) {
        this.decos = this.decos.map(update.changes);
        const uploadFacet = this.view.state.facet(files_upload_facet_1.FileUploadHandlerFacet);
        const changes = [];
        for (const tr of update.transactions) {
            for (const eff of tr.effects) {
                if (eff.is(effects_1.AddUploadWidgetEffect)) {
                    this.decos = this.decos.update({
                        add: Array.from(eff.value.files).map((file, index) => {
                            const { widget } = new FileUploadPresenter({
                                file,
                                view: this.view,
                                uploader: uploadFacet.fn,
                                needDimmensionsForImages: Boolean(uploadFacet.imgWithDimms),
                                widget: new FileUploadWidget((0, utils_1.uniqueId)('__file_widget_id')),
                            });
                            return view_1.Decoration.widget({
                                widget,
                                side: 1,
                                __file_widget_id: widget.selfId,
                            }).range(eff.value.pos + index);
                        }),
                    });
                }
                if (eff.is(effects_1.RemoveUploadWidgetEffect)) {
                    this.decos = this.decos.update({
                        filter: (from, to, deco) => {
                            if (deco.spec.__file_widget_id !== eff.value.id)
                                return true;
                            if (eff.value.markup) {
                                changes.push({ from, to, insert: eff.value.markup });
                            }
                            return false;
                        },
                    });
                }
            }
        }
        if (changes.length) {
            setTimeout(() => {
                this.view.dispatch({ changes });
            });
        }
    }
}, {
    decorations: (v) => v.decos,
    eventHandlers: {
        paste(event, view) {
            if (!event.clipboardData)
                return false;
            const files = (0, utils_1.getTransferFiles)(event.clipboardData);
            if (!files)
                return false;
            const { from, to } = view.state.selection.main;
            view.dispatch({
                selection: { anchor: from },
                effects: effects_1.AddUploadWidgetEffect.of({ files, pos: from }),
                changes: { from, to, insert: ' '.repeat(files.length) },
            });
            event.preventDefault();
            return true;
        },
        drop(event, view) {
            if (!event.dataTransfer)
                return false;
            const files = (0, utils_1.getTransferFiles)(event.dataTransfer);
            if (!files)
                return false;
            const pos = this.view.posAtCoords(event, false);
            view.dispatch({
                selection: { anchor: pos },
                effects: effects_1.AddUploadWidgetEffect.of({ files, pos }),
                changes: { from: pos, insert: ' '.repeat(files.length) },
            });
            event.preventDefault();
            return true;
        },
    },
});
