"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchPanelPlugin = void 0;
const search_1 = require("@codemirror/search");
const view_1 = require("@codemirror/view");
const lodash_1 = require("../../../lodash");
const react_facet_1 = require("../react-facet");
const SearchPopup_1 = require("./view/SearchPopup");
const INPUT_DELAY = 200;
const SearchPanelPlugin = (params) => view_1.ViewPlugin.fromClass(class {
    constructor(view) {
        var _a, _b;
        this.searchQuery = {
            search: '',
            caseSensitive: false,
            wholeWord: false,
        };
        this.view = view;
        this.anchor = null;
        this.renderer = null;
        this.params = params;
        this.receiver = params.receiver;
        this.handleClose = this.handleClose.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.handleSearchNext = this.handleSearchNext.bind(this);
        this.handleSearchPrev = this.handleSearchPrev.bind(this);
        this.handleSearchConfigChange = this.handleSearchConfigChange.bind(this);
        this.handleEditorModeChange = this.handleEditorModeChange.bind(this);
        this.setViewSearchWithDelay = (0, lodash_1.debounce)(this.setViewSearch, (_a = this.params.inputDelay) !== null && _a !== void 0 ? _a : INPUT_DELAY);
        (_b = this.receiver) === null || _b === void 0 ? void 0 : _b.on('change-editor-mode', this.handleEditorModeChange);
    }
    update(update) {
        var _a;
        const isPanelOpen = (0, search_1.searchPanelOpen)(update.state);
        if (isPanelOpen && !this.renderer) {
            this.anchor = document.querySelector(this.params.anchorSelector);
            this.renderer = this.view.state
                .facet(react_facet_1.ReactRendererFacet)
                .createItem('cm-search', () => (0, SearchPopup_1.renderSearchPopup)({
                open: true,
                anchor: this.anchor,
                onChange: this.handleChange,
                onClose: this.handleClose,
                onSearchNext: this.handleSearchNext,
                onSearchPrev: this.handleSearchPrev,
                onConfigChange: this.handleSearchConfigChange,
            }));
        }
        else if (!isPanelOpen && this.renderer) {
            (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
            this.renderer = null;
        }
    }
    destroy() {
        var _a, _b;
        (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
        this.renderer = null;
        (_b = this.receiver) === null || _b === void 0 ? void 0 : _b.off('change-editor-mode', this.handleEditorModeChange);
    }
    setViewSearch(config) {
        this.searchQuery = Object.assign(Object.assign({}, this.searchQuery), config);
        const searchQuery = new search_1.SearchQuery(Object.assign({}, this.searchQuery));
        this.view.dispatch({ effects: search_1.setSearchQuery.of(searchQuery) });
    }
    handleEditorModeChange({ mode }) {
        if (mode === 'wysiwyg') {
            (0, search_1.closeSearchPanel)(this.view);
        }
    }
    handleChange(search) {
        this.setViewSearchWithDelay({ search });
    }
    handleClose() {
        this.setViewSearch({ search: '' });
        (0, search_1.closeSearchPanel)(this.view);
    }
    handleSearchNext() {
        (0, search_1.findNext)(this.view);
    }
    handleSearchPrev() {
        (0, search_1.findPrevious)(this.view);
    }
    handleSearchConfigChange({ isCaseSensitive, isWholeWord, }) {
        this.setViewSearch({
            caseSensitive: isCaseSensitive,
            wholeWord: isWholeWord,
        });
    }
}, {
    provide: () => [
        view_1.keymap.of(search_1.searchKeymap),
        (0, search_1.search)({
            createPanel: () => ({
                // Create an empty search panel
                dom: document.createElement('div'),
            }),
        }),
    ],
});
exports.SearchPanelPlugin = SearchPanelPlugin;
