import type { CommonEditor } from '../common';
import { ReactRenderStorage } from '../extensions';
import { type Receiver } from '../utils/event-emitter';
import type { MarkdownEditorMode as EditorMode, MarkdownEditorPreset as EditorPreset, MarkdownEditorOptions } from './types';
export declare type ToolbarActionData = {
    editorMode: EditorMode;
    id: string;
    attrs?: {
        [key: string]: any;
    };
};
export interface EventMap {
    change: null;
    cancel: null;
    submit: null;
    'toolbar-action': ToolbarActionData;
    'change-editor-mode': {
        mode: EditorMode;
    };
    'change-toolbar-visibility': {
        visible: boolean;
    };
    'change-split-mode-enabled': {
        splitModeEnabled: boolean;
    };
}
export interface Editor extends Receiver<EventMap>, CommonEditor {
    readonly currentMode: EditorMode;
    readonly toolbarVisible: boolean;
    setEditorMode(mode: EditorMode, opts?: SetEditorModeOptions): void;
    moveCursor(position: 'start' | 'end' | {
        line: number;
    }): void;
}
declare type SetEditorModeOptions = Pick<ChangeEditorModeOptions, 'emit'>;
export declare type ChangeEditorModeOptions = {
    mode: EditorMode;
    reason: 'error-boundary' | 'settings' | 'manually';
    emit?: boolean;
};
export declare type EditorOptions = Pick<MarkdownEditorOptions, 'md' | 'initial' | 'handlers' | 'experimental' | 'markupConfig' | 'wysiwygConfig'> & {
    renderStorage: ReactRenderStorage;
    preset: EditorPreset;
};
export {};
