import React, { useCallback } from 'react';
import { Popup } from '@gravity-ui/uikit';
import { FileForm } from '../../../forms/FileForm';
import { i18n } from '../../../i18n/forms';
import { useBooleanState } from '../../../react-utils/hooks';
import { useToaster } from '../../../react-utils/toaster';
import { batchUploadFiles } from '../../../utils/upload';
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
export const ToolbarFilePopup = ({ className, hide, anchorRef, onSubmit, focus, onClick, uploadHandler, onSuccessUpload, }) => {
    const toaster = useToaster();
    const [loading, showLoading, hideLoading] = useBooleanState(false);
    const handleCancel = useCallback(() => {
        hide();
        focus();
    }, [focus, hide]);
    return (React.createElement(Popup, { open: true, onClose: handleCancel, anchorRef: anchorRef, placement: placement, className: className },
        React.createElement(FileForm, { autoFocus: true, onAttach: uploadHandler &&
                ((files) => {
                    showLoading();
                    batchUploadFiles(files, uploadHandler).then((res) => {
                        hideLoading();
                        hide();
                        onSuccessUpload === null || onSuccessUpload === void 0 ? void 0 : onSuccessUpload(res);
                    }, (error) => {
                        hideLoading();
                        toaster.add({
                            theme: 'danger',
                            name: 'toolbar_file_upload',
                            title: i18n('file_upload_failed'),
                            content: String(error),
                        });
                    });
                }), onCancel: handleCancel, onSubmit: (data) => {
                hide();
                focus();
                onSubmit(data);
                onClick === null || onClick === void 0 ? void 0 : onClick('addFile');
            }, loading: loading })));
};
