import { useLayoutEffect, useMemo } from 'react';
import { ReactRenderStorage } from '../extensions';
import { logger } from '../logger';
import { EditorImpl } from './Editor';
import { BundlePreset } from './wysiwyg-preset';
// [major] TODO: remove generic type
export function useMarkdownEditor(props, deps = []) {
    const editor = useMemo(() => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
        const { md = {}, initial = {}, handlers = {}, experimental = {}, markupConfig = {}, wysiwygConfig = {}, } = props;
        const breaks = (_a = md.breaks) !== null && _a !== void 0 ? _a : props.breaks;
        const preset = (_b = props.preset) !== null && _b !== void 0 ? _b : 'full';
        const renderStorage = new ReactRenderStorage();
        const needToSetDimensionsForUploadedImages = (_c = experimental.needToSetDimensionsForUploadedImages) !== null && _c !== void 0 ? _c : props.needToSetDimensionsForUploadedImages;
        const extensions = (builder) => {
            var _a;
            const extensionOptions = (_a = wysiwygConfig.extensionOptions) !== null && _a !== void 0 ? _a : props.extensionOptions;
            builder.use(BundlePreset, Object.assign(Object.assign({}, extensionOptions), { preset, reactRenderer: renderStorage, onCancel: () => {
                    editor.emit('cancel', null);
                    return true;
                }, onSubmit: () => {
                    editor.emit('submit', null);
                    return true;
                }, mdBreaks: breaks, fileUploadHandler: props.fileUploadHandler, needToSetDimensionsForUploadedImages }));
            {
                const extraExtensions = wysiwygConfig.extensions || props.extraExtensions;
                if (extraExtensions) {
                    builder.use(extraExtensions, props.extensionOptions);
                }
            }
        };
        return new EditorImpl(Object.assign(Object.assign({}, props), { preset,
            renderStorage, md: Object.assign(Object.assign({}, md), { breaks, html: (_d = md.html) !== null && _d !== void 0 ? _d : props.allowHTML, linkify: (_e = md.linkify) !== null && _e !== void 0 ? _e : props.linkify, linkifyTlds: (_f = md.linkifyTlds) !== null && _f !== void 0 ? _f : props.linkifyTlds }), initial: Object.assign(Object.assign({}, initial), { markup: (_g = initial.markup) !== null && _g !== void 0 ? _g : props.initialMarkup, mode: (_h = initial.mode) !== null && _h !== void 0 ? _h : props.initialEditorMode, toolbarVisible: (_j = initial.toolbarVisible) !== null && _j !== void 0 ? _j : props.initialToolbarVisible, splitModeEnabled: (_k = initial.splitModeEnabled) !== null && _k !== void 0 ? _k : props.initialSplitModeEnabled }), handlers: Object.assign(Object.assign({}, handlers), { uploadFile: (_l = handlers.uploadFile) !== null && _l !== void 0 ? _l : props.fileUploadHandler }), experimental: Object.assign(Object.assign({}, experimental), { needToSetDimensionsForUploadedImages, prepareRawMarkup: (_m = experimental.prepareRawMarkup) !== null && _m !== void 0 ? _m : props.prepareRawMarkup, beforeEditorModeChange: (_o = experimental.beforeEditorModeChange) !== null && _o !== void 0 ? _o : props.experimental_beforeEditorModeChange }), markupConfig: Object.assign(Object.assign({}, markupConfig), { splitMode: (_p = markupConfig.splitMode) !== null && _p !== void 0 ? _p : props.splitMode, renderPreview: (_q = markupConfig.renderPreview) !== null && _q !== void 0 ? _q : props.renderPreview, extensions: (_r = markupConfig.extensions) !== null && _r !== void 0 ? _r : props.extraMarkupExtensions }), wysiwygConfig: Object.assign(Object.assign({}, wysiwygConfig), { extensions, escapeConfig: (_s = wysiwygConfig.escapeConfig) !== null && _s !== void 0 ? _s : props.escapeConfig }) }));
    }, deps);
    useLayoutEffect(() => {
        function onToolbarAction({ editorMode, id }) {
            logger.action({ mode: editorMode, source: 'toolbar', action: id });
        }
        editor.on('toolbar-action', onToolbarAction);
        return () => {
            editor.off('toolbar-action', onToolbarAction);
            editor.destroy();
        };
    }, [editor]);
    return editor;
}
