import { logger } from '../../../../logger';
import { FilesBatchUploadProcess } from '../../../behavior/utils/upload';
import { imageType } from '../../../markdown';
import { IMG_MAX_HEIGHT } from '../const';
import { createImageNode, loadImage } from '../utils';
import { ImageSkeletonDescriptor } from './skeleton';
export class ImagesUploadProcess extends FilesBatchUploadProcess {
    constructor(view, files, uploadHandler, position, opts) {
        super(view, files, uploadHandler);
        this.initPosition = position;
        this.createImage = createImageNode(imageType(this.view.state.schema), opts);
    }
    async createSkeleton() {
        return new ImageSkeletonDescriptor(this.initPosition, await getSkeletonSize(this.files));
    }
    async createPMNode(res) {
        return this.createImage(res);
    }
}
async function getSkeletonSize(files) {
    const skeletonSize = { width: '300', height: '200' };
    if (files.length === 1) {
        try {
            const size = await loadImage(files[0]).then(calcSkeletonSize);
            skeletonSize.width = String(size.width);
            skeletonSize.height = String(size.height);
        }
        catch (err) {
            logger.error(err);
        }
    }
    return skeletonSize;
}
function calcSkeletonSize(img) {
    const { width, height } = img;
    if (height <= IMG_MAX_HEIGHT)
        return { width, height };
    const ratio = IMG_MAX_HEIGHT / height; // ratio<1
    return { height: IMG_MAX_HEIGHT, width: width * ratio };
}
