import { SearchQuery, closeSearchPanel, findNext, findPrevious, search, searchKeymap, searchPanelOpen, setSearchQuery, } from '@codemirror/search';
import { ViewPlugin, keymap } from '@codemirror/view';
import { debounce } from '../../../lodash';
import { ReactRendererFacet } from '../react-facet';
import { renderSearchPopup } from './view/SearchPopup';
const INPUT_DELAY = 200;
export const SearchPanelPlugin = (params) => ViewPlugin.fromClass(class {
    constructor(view) {
        var _a, _b;
        this.searchQuery = {
            search: '',
            caseSensitive: false,
            wholeWord: false,
        };
        this.view = view;
        this.anchor = null;
        this.renderer = null;
        this.params = params;
        this.receiver = params.receiver;
        this.handleClose = this.handleClose.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.handleSearchNext = this.handleSearchNext.bind(this);
        this.handleSearchPrev = this.handleSearchPrev.bind(this);
        this.handleSearchConfigChange = this.handleSearchConfigChange.bind(this);
        this.handleEditorModeChange = this.handleEditorModeChange.bind(this);
        this.setViewSearchWithDelay = debounce(this.setViewSearch, (_a = this.params.inputDelay) !== null && _a !== void 0 ? _a : INPUT_DELAY);
        (_b = this.receiver) === null || _b === void 0 ? void 0 : _b.on('change-editor-mode', this.handleEditorModeChange);
    }
    update(update) {
        var _a;
        const isPanelOpen = searchPanelOpen(update.state);
        if (isPanelOpen && !this.renderer) {
            this.anchor = document.querySelector(this.params.anchorSelector);
            this.renderer = this.view.state
                .facet(ReactRendererFacet)
                .createItem('cm-search', () => renderSearchPopup({
                open: true,
                anchor: this.anchor,
                onChange: this.handleChange,
                onClose: this.handleClose,
                onSearchNext: this.handleSearchNext,
                onSearchPrev: this.handleSearchPrev,
                onConfigChange: this.handleSearchConfigChange,
            }));
        }
        else if (!isPanelOpen && this.renderer) {
            (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
            this.renderer = null;
        }
    }
    destroy() {
        var _a, _b;
        (_a = this.renderer) === null || _a === void 0 ? void 0 : _a.remove();
        this.renderer = null;
        (_b = this.receiver) === null || _b === void 0 ? void 0 : _b.off('change-editor-mode', this.handleEditorModeChange);
    }
    setViewSearch(config) {
        this.searchQuery = Object.assign(Object.assign({}, this.searchQuery), config);
        const searchQuery = new SearchQuery(Object.assign({}, this.searchQuery));
        this.view.dispatch({ effects: setSearchQuery.of(searchQuery) });
    }
    handleEditorModeChange({ mode }) {
        if (mode === 'wysiwyg') {
            closeSearchPanel(this.view);
        }
    }
    handleChange(search) {
        this.setViewSearchWithDelay({ search });
    }
    handleClose() {
        this.setViewSearch({ search: '' });
        closeSearchPanel(this.view);
    }
    handleSearchNext() {
        findNext(this.view);
    }
    handleSearchPrev() {
        findPrevious(this.view);
    }
    handleSearchConfigChange({ isCaseSensitive, isWholeWord, }) {
        this.setViewSearch({
            caseSensitive: isCaseSensitive,
            wholeWord: isWholeWord,
        });
    }
}, {
    provide: () => [
        keymap.of(searchKeymap),
        search({
            createPanel: () => ({
                // Create an empty search panel
                dom: document.createElement('div'),
            }),
        }),
    ],
});
