import { AllSelection, NodeSelection, TextSelection } from 'prosemirror-state';
NodeSelection.prototype.selectionName = 'NodeSelection';
TextSelection.prototype.selectionName = 'TextSelection';
AllSelection.prototype.selectionName = 'AllSelection';
export const isTextSelection = (selection) => selection.selectionName === TextSelection.prototype.selectionName;
export const isNodeSelection = (selection) => selection.selectionName === NodeSelection.prototype.selectionName;
// ts broke down when use "selection is AllSelection" return type
// maybe because AllSelection has same class type with different constructor
export const isWholeSelection = (selection) => selection.selectionName === AllSelection.prototype.selectionName;
export const get$Cursor = (selection) => {
    return isTextSelection(selection) ? selection.$cursor : null;
};
export function get$CursorAtBlockStart(selection) {
    const $cursor = get$Cursor(selection);
    if (!$cursor || $cursor.parentOffset > 0 || $cursor.parent.isInline)
        return null;
    return $cursor;
}
export const equalNodeType = function equalNodeType(nodeType, node) {
    return (Array.isArray(nodeType) && nodeType.indexOf(node.type) > -1) || node.type === nodeType;
};
export const findSelectedNodeOfType = (nodeType) => {
    return function (selection) {
        if (isNodeSelection(selection)) {
            const node = selection.node, $from = selection.$from;
            if (equalNodeType(nodeType, node)) {
                return { node: node, pos: $from.pos, depth: $from.depth };
            }
        }
        return null;
    };
};
